# Filename: test_online.py
# pylint: disable=locally-disabled,C0111,R0904,C0301,C0103,W0212
from km3pipe.testing import TestCase, patch, skip, data_path
import sys
import numpy as np
from km3pipe.io.online import EventPump, TimeslicePump, SummaryslicePump
from km3pipe.io.hdf5 import HDF5Pump
from os.path import join, dirname, abspath

__author__ = "Tamas Gal"
__copyright__ = "Copyright 2018, Tamas Gal and the KM3NeT collaboration."
__credits__ = []
__license__ = "MIT"
__maintainer__ = "Tamas Gal, Johannes Schumann"
__email__ = "tgal@km3net.de"
__status__ = "Development"


ONLINE_TEST_FILE = data_path("online/events_sample.root")
ONLINE_TEST_FILE_H5 = data_path("online/events_sample.h5")


class TestEventPump(TestCase):
    def setUp(self):
        self.jpp_pump = EventPump(filename=ONLINE_TEST_FILE)
        self.h5_pump = HDF5Pump(filename=ONLINE_TEST_FILE_H5)

    def tearDown(self):
        self.jpp_pump.finish()
        self.h5_pump.finish()

    def test_event_info(self):
        n = self.jpp_pump.n_events
        for i in range(n):
            h5blob = self.h5_pump[i]
            jppblob = self.jpp_pump[i]
            for k in h5blob["EventInfo"].dtype.names:
                if k not in jppblob["EventInfo"].dtype.names:
                    continue
                ref_value = h5blob["EventInfo"][k][0]
                test_value = jppblob["EventInfo"][k][0]
                if isinstance(ref_value, float):
                    if np.isnan(ref_value):
                        assert np.isnan(test_value) == np.isnan(ref_value)
                    else:
                        self.assertAlmostEqual(test_value, ref_value)
                else:
                    assert ref_value == test_value

    @skip
    def test_hit_info(self):
        n = self.jpp_pump.n_events
        for i in range(n):
            h5blob = self.h5_pump[i]
            jppblob = self.jpp_pump[i]
            for k in h5blob["Hits"].dtype.names:
                ref_value = h5blob["Hits"][k][0]
                test_value = jppblob["Hits"][k][0]
                if isinstance(ref_value, float):
                    if np.isnan(ref_value):
                        assert np.isnan(test_value)
                    else:
                        self.assertAlmostEqual(test_value, ref_value)
                else:
                    assert ref_value == test_value


class TestTimeslicePump(TestCase):
    def setUp(self):
        self.jpp_fname = ONLINE_TEST_FILE
        self.timeslice_pump = TimeslicePump(
            filename=self.jpp_fname, stream="L1"
        )

    def test_timeslice_info(self):
        blob = self.timeslice_pump[0]
        tsinfo = blob["TimesliceInfo"]
        assert tsinfo.frame_index == 366
        assert tsinfo.slice_id == 0
        assert tsinfo.timestamp == 1575979236
        assert tsinfo.nanoseconds == 600000000
        assert tsinfo.n_frames == 69

    def test_timeslice_hits(self):
        blob = self.timeslice_pump[0]
        l1hits = blob["L1Hits"]
        counts = [
            1421, 1272, 1533, 1209, 1311, 1450, 1102, 1494, 1410, 1390, 1225,
            1419, 1474, 1452, 1589, 1362, 1577, 1598, 1891, 1584, 1571, 1523,
            1696, 1719, 1540, 1578, 1656, 1569, 1422, 1401, 1262
        ]
        tot = [
            216, 681, 1222, 1580, 1484, 1221, 940, 887, 908, 950, 844, 736,
            717, 661, 752, 716, 791, 946, 1089, 1220, 1484, 1786, 2265, 2539,
            2932, 3132, 3201, 2901, 2403, 1654, 1056, 566, 297, 167, 94, 74,
            82, 60, 50, 37, 30, 24, 24, 20, 19, 12, 15, 15, 13, 8, 5, 6, 9, 9,
            7, 9, 11, 6, 5, 10, 7, 4, 7, 2, 7, 1, 4, 5, 5, 3, 3, 3, 2, 3, 2, 3,
            1, 4, 3, 4, 1, 1, 2, 2, 1, 2, 3, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 1,
            1, 1, 1, 1, 1
        ]
        dom_id = [
            774, 244, 602, 935, 533, 878, 862, 699, 760, 1058, 625, 641, 621,
            679, 912, 461, 925, 633, 754, 701, 767, 784, 812, 299, 605, 1010,
            226, 892, 785, 864, 707, 540, 749, 772, 172, 748, 773, 585, 278,
            656, 852, 685, 1041, 473, 639, 695, 837, 830, 656, 676, 486, 647,
            533, 716, 649, 599, 502, 271, 629, 769, 613, 784, 705, 600, 984,
            682, 243, 583
        ]
        self.assertListEqual(
            counts,
            np.unique(l1hits.channel_id, return_counts=True)[1].tolist()
        )
        self.assertListEqual(
            tot,
            np.unique(l1hits.tot, return_counts=True)[1].tolist()
        )
        self.assertListEqual(
            dom_id,
            np.unique(l1hits.dom_id, return_counts=True)[1].tolist()
        )


class TestSummaryslicePump(TestCase):
    def setUp(self):
        self.pump = SummaryslicePump(filename=ONLINE_TEST_FILE)

    def test_summaryslice_info(self):
        sum_slice_info = self.pump[0]['SummarysliceInfo']
        assert sum_slice_info.frame_index == 183
        assert sum_slice_info.timestamp == 1575979218
        assert sum_slice_info.nanoseconds == 300000000 / 16
        assert sum_slice_info.n_frames == 2

    def test_summaryslice_field_len(self):
        sum_slice = self.pump[0]['Summaryslice']
        dom_ids = [808447180, 808488895]
        self.assertListEqual(dom_ids, list(sum_slice.keys()))
        for i in range(2):
            assert len(sum_slice[dom_ids[i]]['rates']) == 31
            assert len(sum_slice[dom_ids[i]]['fifos']) == 31
            assert len(sum_slice[dom_ids[i]]['hrvs']) == 31

    def test_summaryslice_ctnt(self):
        sum_slice = self.pump[0]['Summaryslice']
        dom_ids = [808447180, 808488895]
        assert sum_slice[dom_ids[0]]['max_sequence_number'] == 26
        assert sum_slice[dom_ids[0]]['n_udp_packets'] == 27
        assert sum_slice[dom_ids[0]]['has_udp_trailer']
        assert not sum_slice[dom_ids[0]]['fifo_status']
        self.assertAlmostEqual(
            sum_slice[dom_ids[0]]['rates'][6],
            6586.,
        )
        self.assertAlmostEqual(sum_slice[dom_ids[1]]['rates'][4], 5752.0)



__author__ = 'tamasgal'

from km3pipe import Pipeline, Module
from km3pipe.io import DAQPump


class DAQEventPrinter(Module):
    def process(self, blob):
        try:
            print(blob['DAQEvent'])
        except KeyError:
            pass
        return blob


class DAQSummaryslicePrinter(Module):
    def process(self, blob):
        try:
            print(blob['DAQSummaryslice'])
        except KeyError:
            pass
        return blob


class MeanHits(Module):
    def configure(self):
        self.hits = []

    def process(self, blob):
        try:
            event = blob['DAQEvent']
            self.hits.append(event.n_snapshot_hits)
        except KeyError:
            pass
        return blob

    def finish(self):
        mean_hits = sum(self.hits) / len(self.hits)
        print(
            "Number of entries: {0}\nMean hits: {1}".format(
                len(self.hits), mean_hits
            )
        )


class MeanRates(Module):
    def configure(self):
        self.rates = {}

    def process(self, blob):
        try:
            summaryslice = blob['DAQSummaryslice']
            print(summaryslice.summary_frames)
        except KeyError:
            pass
        return blob

    def finish(self):
        pass


pipeline = Pipeline()
pipeline.attach(DAQPump, 'daq_pump', filename='dump.dat')
# pipeline.attach(DAQEventPrinter, 'moo')
# pipeline.attach(DAQSummaryslicePrinter, 'summaryslice_printer')
# pipeline.attach(MeanRates, 'mean_rates')
pipeline.attach(MeanHits, 'mean_hits')
pipeline.drain()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ts=4 sw=4 et
"""
=============
Log.io Module
=============

A module to send live log messages to a log.io server.
"""

# Author: Tamas Gal <tgal@km3net.de>
# License: MIT
import socket

from km3pipe import Pipeline, Module
from km3pipe.pumps import CHPump


class LogIO(Module):
    def configure(self):
        url = 'pi2089.physik.uni-erlangen.de'
        port = 28777

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((url, port))

    def process(self, blob):
        data = blob['CHData']
        log_level = 'info'
        if "ERROR" in data:
            log_level = 'error'
        if "WARNING" in data:
            log_level = 'warning'
        source = "Other"
        if " F0" in data:
            source = "DataFilter"
        if " Q0" in data:
            source = "DataQueue"
        if " W0" in data:
            source = "DataWriter"
        self.sock.send(
            "+log|{0}|Portopalo DAQ|{1}|{2}\r\n".format(
                source, log_level, data
            )
        )
        return blob

    def finish(self):
        self.sock.close()


pipe = Pipeline()
pipe.attach(
    CHPump,
    host='127.0.0.1',
    port=5553,
    tags='MSG',
    timeout=60 * 60 * 24,
    max_queue=500
)
pipe.attach(LogIO)
pipe.drain()



"""Open a file with real data and estimate the DOM rates.
"""

from km3pipe import Module, Pipeline
from km3pipe.io.aanet import AanetPump


class RateEstimator(Module):
    def process(self, blob):
        hits = blob["Hits"]
        doms = {h.dom_id for h in hits}
        hit_times = [h.t for h in hits]
        event_length = max(hit_times) - min(hit_times)
        n_doms = len(doms)
        n_hits = len(hits)

        print("Active DOMs: {0}".format(doms))
        print("Number of active DOMs: {0}".format(n_doms))
        print("Event length: {0} ns".format(event_length))
        print("Number of hits: {0}".format(n_hits))

        rate_per_dom = 1e6 / (event_length / (n_hits / n_doms))

        print("Estimated rate per DOM: {0} kHz".format(rate_per_dom))


pipe = Pipeline()
pipe.attach(AanetPump, filename='KM3NeT_00000007_00001000.root')
pipe.attach(RateEstimator)
pipe.drain(10)



import urwid
import random


class ItemWidget(urwid.WidgetWrap):
    def __init__(self, id, description):
        self.id = id
        self.content = 'item %s: %s...' % (str(id), description[:25])
        self.item = [
            (
                'fixed', 15,
                urwid.Padding(
                    urwid.AttrWrap(
                        urwid.Text('item %s' % str(id)), 'body', 'focus'
                    ),
                    left=2
                )
            ),
            urwid.AttrWrap(urwid.Text('%s' % description), 'body', 'focus'),
        ]
        w = urwid.Columns(self.item)
        self.__super.__init__(w)

    def selectable(self):
        return True

    def keypress(self, size, key):
        return key


def main():

    palette = [
        ('body', 'dark cyan', '', 'standout'),
        ('focus', 'dark red', '', 'standout'),
        ('head', 'light red', 'black'),
    ]

    lorem = [
        'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
        'Sed sollicitudin, nulla id viverra pulvinar.',
        'Cras a magna sit amet felis fringilla lobortis.',
    ]

    def keystroke(input):
        if input in ('q', 'Q'):
            raise urwid.ExitMainLoop()

        if input is 'enter':
            focus = listbox.get_focus()[0].content
            view.set_header(
                urwid.AttrWrap(
                    urwid.Text('selected: %s' % str(focus)), 'head'
                )
            )

    items = []
    for i in range(100):
        items.append(ItemWidget(i, random.choice(lorem)))

    header = urwid.AttrMap(urwid.Text('selected:'), 'head')
    listbox = urwid.ListBox(urwid.SimpleListWalker(items))
    view = urwid.Frame(urwid.AttrWrap(listbox, 'body'), header=header)
    loop = urwid.MainLoop(view, palette, unhandled_input=keystroke)
    loop.run()


if __name__ == '__main__':
    main()



# Filename: testing.py
"""
Common unit testing support for km3pipe.

"""
import sys
from functools import wraps
from unittest import TestCase    # noqa
from mock import MagicMock    # noqa
from mock import Mock    # noqa
from mock import patch    # noqa

from numpy.testing import assert_allclose    # noqa
import pytest    # noqa

from km3net_testdata import data_path

skip = pytest.mark.skip
skipif = pytest.mark.skipif

__author__ = "Tamas Gal"
__copyright__ = "Copyright 2016, Tamas Gal and the KM3NeT collaboration."
__credits__ = []
__license__ = "MIT"
__maintainer__ = "Tamas Gal"
__email__ = "tgal@km3net.de"
__status__ = "Development"



# Filename: plot.py
# -*- coding: utf-8 -*-
# pylint: disable=locally-disabled
"""
A collection of plotting functions and modules.

"""
from collections import Counter
from datetime import datetime
import os
import multiprocessing as mp
import time

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt    # noqa
import matplotlib.ticker as ticker
from matplotlib import pylab    # noqa

import km3pipe as kp    # noqa
import km3pipe.style    # noqa

from km3modules.hits import count_multiplicities


def plot_dom_parameters(
    data,
    detector,
    filename,
    label,
    title,
    vmin=0.0,
    vmax=10.0,
    cmap='RdYlGn_r',
    under='deepskyblue',
    over='deeppink',
    underfactor=1.0,
    overfactor=1.0,
    missing='lightgray',
    hide_limits=False
):
    """Creates a plot in the classical monitoring.km3net.de style.

    Parameters
    ----------
    data: dict((du, floor) -> value)
    detector: km3pipe.hardware.Detector() instance
    filename: filename or filepath
    label: str
    title: str
    underfactor: a scale factor for the points used for underflow values
    overfactor: a scale factor for the points used for overflow values
    hide_limits: do not show under/overflows in the plot

    """
    x, y, _ = zip(*detector.doms.values())
    fig, ax = plt.subplots(figsize=(10, 6))
    cmap = plt.get_cmap(cmap)
    cmap.set_over(over, 1.0)
    cmap.set_under(under, 1.0)

    m_size = 100
    scatter_args = {
        'edgecolors': 'None',
        'vmin': vmin,
        'vmax': vmax,
    }
    sc_inactive = ax.scatter(
        x, y, c=missing, label='missing', s=m_size * 0.9, **scatter_args
    )

    xa, ya = map(np.array, zip(*data.keys()))
    zs = np.array(list(data.values()))
    in_range_idx = np.logical_and(zs >= vmin, zs <= vmax)
    sc = ax.scatter(
        xa[in_range_idx],
        ya[in_range_idx],
        c=zs[in_range_idx],
        cmap=cmap,
        s=m_size,
        **scatter_args
    )
    if not hide_limits:
        under_idx = zs < vmin
        ax.scatter(
            xa[under_idx],
            ya[under_idx],
            c=under,
            label='< {0}'.format(vmin),
            s=m_size * underfactor,
            **scatter_args
        )
        over_idx = zs > vmax
        ax.scatter(
            xa[over_idx],
            ya[over_idx],
            c=over,
            label='> {0}'.format(vmax),
            s=m_size * overfactor,
            **scatter_args
        )

    cb = plt.colorbar(sc)
    cb.set_label(label)

    ax.set_title(
        "{0}\n{1} UTC".format(title,
                              datetime.utcnow().strftime("%c"))
    )
    ax.set_xlabel("DU")
    ax.set_ylabel("DOM")
    ax.set_ylim(-2)
    ax.set_yticks(range(1, 18 + 1))
    major_locator = pylab.MaxNLocator(integer=True)
    sc_inactive.axes.xaxis.set_major_locator(major_locator)

    ax.legend(
        bbox_to_anchor=(0., -.16, 1., .102),
        loc=1,
        ncol=2,
        mode="expand",
        borderaxespad=0.
    )

    fig.tight_layout()

    plt.savefig(filename, dpi=120, bbox_inches="tight")
    plt.close('all')


def make_dom_map(pmt_directions, values, nside=512, d=0.2, smoothing=0.1):
    """Create a mollweide projection of a DOM with given PMTs.

    The output can be used to call the `healpy.mollview` function.
    """
    import healpy as hp
    discs = [hp.query_disc(nside, dir, 0.2) for dir in pmt_directions]
    npix = hp.nside2npix(nside)
    pixels = np.zeros(npix)
    for disc, value in zip(discs, values):
        for d in disc:
            pixels[d] = value
    if smoothing > 0:
        return hp.sphtfunc.smoothing(pixels, fwhm=smoothing, iter=1)
    return pixels


class IntraDOMCalibrationPlotter(kp.Module):
    def configure(self):
        self.plots_path = self.get('plots_path', default=os.getcwd())
        self.data_path = self.get('data_path', default=os.getcwd())
        self.det_oid = self.require('det_oid')
        self.clbmap = kp.db.CLBMap(self.det_oid)

    def process(self, blob):
        calibration = blob["IntraDOMCalibration"]
        for process in (self.create_plot, self.save_hdf5):
            proc = mp.Process(target=process, args=(calibration, ))
            proc.daemon = True
            proc.start()
            proc.join()
        return blob

    def create_plot(self, calibration):
        print("Creating plot...")
        fig, axes = plt.subplots(
            6, 3, figsize=(16, 20), sharex=True, sharey=True
        )
        sorted_dom_ids = sorted(
            calibration.keys(),
            key=lambda d:
            (self.clbmap.dom_ids[d].du, self.clbmap.dom_ids[d].floor)
        )    # by DU and FLOOR, note that DET OID is needed!
        for ax, dom_id in zip(axes.flatten(), sorted_dom_ids):
            calib = calibration[dom_id]
            ax.plot(np.cos(calib['angles']), calib["means"], '.')
            ax.plot(np.cos(calib['angles']), calib["corrected_means"], '.')
            du = self.clbmap.dom_ids[dom_id].du
            floor = self.clbmap.dom_ids[dom_id].floor
            ax.set_title(
                "{0} - {1}".format("DU{}-DOM{}".format(du, floor), dom_id)
            )
            ax.set_ylim((-10, 10))
        plt.suptitle("{0} UTC".format(datetime.utcnow().strftime("%c")))
        plt.savefig(
            os.path.join(self.plots_path, "intradom.png"), bbox_inches='tight'
        )
        plt.close('all')

        fig, axes = plt.subplots(
            6, 3, figsize=(16, 20), sharex=True, sharey=True
        )
        for ax, dom_id in zip(axes.flatten(), sorted_dom_ids):
            calib = calibration[dom_id]
            ax.plot(np.cos(calib['angles']), calib["rates"], '.')
            ax.plot(np.cos(calib['angles']), calib["corrected_rates"], '.')
            du = self.clbmap.dom_ids[dom_id].du
            floor = self.clbmap.dom_ids[dom_id].floor
            ax.set_title(
                "{0} - {1}".format("DU{}-DOM{}".format(du, floor), dom_id)
            )
            ax.set_ylim((0, 10))
        plt.suptitle("{0} UTC".format(datetime.utcnow().strftime("%c")))
        plt.savefig(
            os.path.join(self.plots_path, "angular_k40rate_distribution.png"),
            bbox_inches='tight'
        )
        plt.close('all')

    def save_hdf5(self, calibration):
        print("Saving calibration information...")
        store = pd.HDFStore(
            os.path.join(self.data_path, 'k40calib.h5'), mode='a'
        )
        now = int(time.time())
        timestamps = (now, ) * 31
        for dom_id, calib in calibration.items():
            tdc_channels = range(31)
            t0s = calib['opt_t0s'].x
            dom_ids = (dom_id, ) * 31
            df = pd.DataFrame({
                'timestamp': timestamps,
                'dom_id': dom_ids,
                'tdc_channel': tdc_channels,
                't0s': t0s
            })
            store.append('t0s', df, format='table', data_columns=True)
        store.close()


def ztplot(
    hits,
    filename=None,
    title=None,
    max_z=None,
    figsize=(16, 8),
    n_dus=4,
    ytick_distance=200,
    max_multiplicity_entries=10,
    grid_lines=[]
):
    """Creates a ztplot like shown in the online monitoring"""
    fontsize = 16

    dus = set(hits.du)

    if n_dus is not None:
        dus = [c[0] for c in Counter(hits.du).most_common(n_dus)]
        mask = [du in dus for du in hits.du]
        hits = hits[mask]

    dus = sorted(dus)
    doms = set(hits.dom_id)

    hits = hits.append_columns('multiplicity',
                               np.ones(len(hits))).sorted(by='time')

    if max_z is None:
        max_z = int(np.ceil(np.max(hits.pos_z) / 100.0)) * 100 * 1.05

    for dom in doms:
        dom_hits = hits[hits.dom_id == dom]
        mltps, m_ids = count_multiplicities(dom_hits.time)
        hits['multiplicity'][hits.dom_id == dom] = mltps

    time_offset = np.min(hits[hits.triggered > 0].time)
    hits.time -= time_offset

    n_plots = len(dus)
    n_cols = int(np.ceil(np.sqrt(n_plots)))
    n_rows = int(n_plots / n_cols) + (n_plots % n_cols > 0)
    marker_fig, marker_axes = plt.subplots()
    # for the marker size hack...
    fig, axes = plt.subplots(
        ncols=n_cols,
        nrows=n_rows,
        sharex=True,
        sharey=True,
        figsize=figsize,
        constrained_layout=True
    )

    axes = [axes] if n_plots == 1 else trim_axes(axes, n_plots)

    for ax, du in zip(axes, dus):
        du_hits = hits[hits.du == du]
        for grid_line in grid_lines:
            ax.axhline(grid_line, lw=1, color='b', ls='--', alpha=0.15)
        trig_hits = du_hits[du_hits.triggered > 0]

        ax.scatter(
            du_hits.time,
            du_hits.pos_z,
            s=du_hits.multiplicity * 30,
            c='#09A9DE',
            label='hit',
            alpha=0.5
        )
        ax.scatter(
            trig_hits.time,
            trig_hits.pos_z,
            s=trig_hits.multiplicity * 30,
            alpha=0.8,
            marker="+",
            c='#FF6363',
            label='triggered hit'
        )
        ax.set_title(
            'DU{0}'.format(int(du)), fontsize=fontsize, fontweight='bold'
        )

        # The only way I could create a legend with matching marker sizes
        max_multiplicity = int(np.max(du_hits.multiplicity))
        markers = list(
            range(
                0, max_multiplicity,
                np.ceil(max_multiplicity / max_multiplicity_entries
                        ).astype(int)
            )
        )
        custom_markers = [
            marker_axes.
            scatter([], [], s=mult * 30, color='#09A9DE', lw=0, alpha=0.5)
            for mult in markers
        ] + [marker_axes.scatter([], [], s=30, marker="+", c='#FF6363')]
        ax.legend(
            custom_markers,
            ['multiplicity'] + ["       %d" % m
                                for m in markers[1:]] + ['triggered'],
            scatterpoints=1,
            markerscale=1,
            loc='lower right',
            frameon=True,
            framealpha=0.7
        )

    for idx, ax in enumerate(axes):
        ax.set_ylim(0, max_z)
        ax.tick_params(labelsize=fontsize)
        ax.yaxis.set_major_locator(ticker.MultipleLocator(ytick_distance))
        xlabels = ax.get_xticklabels()
        for label in xlabels:
            label.set_rotation(45)

        if idx % n_cols == 0:
            ax.set_ylabel('z [m]', fontsize=fontsize)
        if idx >= len(axes) - n_cols:
            ax.set_xlabel('time [ns]', fontsize=fontsize)

    if title is not None:
        plt.suptitle(title, fontsize=fontsize, y=1.05)
    if filename is not None:
        plt.savefig(filename, dpi=120, bbox_inches="tight")

    return fig


def trim_axes(axes, n):
    """little helper to massage the axes list to have correct length..."""
    axes = axes.flat
    for ax in axes[n:]:
        ax.remove()
    return axes[:n]


def cumulative_run_livetime(qtable, kind='runs'):
    """Create a figure which plots the cumulative livetime of runs

    Parameters
    ----------
    qtable: pandas.DataFrame
        A table which has the run number as index and columns for
        'livetime_s', 'timestamp' and 'datetime' (pandas datetime).
    kind: str
        'runs' to plot for each run or 'timeline' to plot based
        on the actual run time.

    Returns
    -------
    matplotlib.Figure
    """
    fig, ax = plt.subplots()

    options = {
        'runs': {
            'xlabel': 'run',
            'xs': qtable.index
        },
        'timeline': {
            'xlabel': None,
            'xs': qtable.datetime
        }
    }

    actual_livetime = np.cumsum(qtable['livetime_s'])
    optimal_livetime = np.cumsum(qtable.timestamp.diff())

    ax.plot(options[kind]['xs'], actual_livetime, label='actual livetime')
    ax.plot(options[kind]['xs'], optimal_livetime, label='100% livetime')
    ax.set_xlabel(options[kind]['xlabel'])
    ax.set_ylabel('time / s')
    ax.legend()

    return fig
